#!/bin/bash
#
# Net Responsibility
#
# Copyright (C) 2007 Scott Severance
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Author:
#	Scott Severance <http://www.scottseverance.us>
#
#
#
# This is the Net Responsibility installer.

if [ `whoami` != root ]; then
  echo "You must run this script as root."
  exit 1
fi

echo "Installing Net Responsibility..."

sharedir=/usr/share/net-responsibility/
confdir=/etc/
initdir=/etc/init.d/
initscript="${initdir}net-responsibility-daemon"
bindir=/usr/local/bin/
 
mkdir -p "$sharedir"
cp -r src/* "$sharedir"
if [ -f "${confdir}net-responsibility.conf" ]; then
  echo "================================================================"
  echo "Found (and keeping) previous configuration. If you want to use"
  echo "the latest configuration file, you can find it at"
  echo "        ${sharedir}net-responsibility.conf"
  echo
  echo "Move it to"
  echo "        ${confdir}net-responsibility.conf"
  echo "to enable it. Be sure to restart the Net Responsibility daemon"
  echo "afterwards."
  echo "================================================================"
  echo
else
  mv "${sharedir}net-responsibility.conf" "$confdir"
fi
mv "${sharedir}net-responsibility-daemon.ctrl" "$initscript"

for i in /etc/rc{2,3,4,5}.d; do
  ln -s "$initscript" "$i/S50net-responsibility-daemon"
done
for i in /etc/rc{0,1,6}.d; do
  ln -s "$initscript" "$i/K50net-responsibility-daemon"
done

ln -s "${sharedir}net-responsibility-daemon" "$bindir"
ln -s "${sharedir}net-responsibility-report" "$bindir"
ln -s "${bindir}net-responsibility-report" /etc/cron.weekly

echo "Copied files."
echo
echo "You must configure Net Responsibility before using it. The"
echo "installer will now start an editor. When you finish, save the file"
echo "and exit the editor in order to finalize the installation."
echo
echo "Press any key to start your editor."
read -n 1

nano "${confdir}net-responsibility.conf"

echo "Starting Net Responsibility"
$initscript start
status=$?
if [ $status -eq 0 ]; then
  echo "Installation complete."
  exit 0
else
  echo "There was an error starting Net Responsibility."
  exit $status
fi





