raise "Can't run this file directly" unless defined? $lib_dir
# Net Responsibility
#
# Copyright (C) 2007 Scott Severance
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Author:
#	Scott Severance <http://www.scottseverance.us>

require 'date'
require 'yaml'

require "#{$lib_dir}my_array"
require "#{$lib_dir}my_date"
require "#{$lib_dir}exceptions"

class Logger
  def initialize(opts)
    logfile = opts.logfile
    daemonize = opts.daemonize
    if !logfile then raise "No logfile given in the config file"; end
    if daemonize then @pid_file = daemonize; end
    @logfile = nil
    @logfile_name = logfile
    @urlsnarf = `which urlsnarf`.chomp
    status = $?.exitstatus
    if status != 0
      case status
        when 1
          raise "Unable to locate urlsnarf."
        else
          raise "which returned an exit status of #{status}"
      end
    end
    @trapped = proc do
      msg = My_Array.new
      msg.push({'type' => 'shutdown', 'method' => 'Signal',
                'time' => DateTime.now.to_s})
      @logfile << msg.to_yaml
      @logfile.close
      if daemonize then File.delete @pid_file; end
      exit!(0)
    end
    @reset = proc do
      msg = My_Array.new
      msg.push({'type' => 'reset', 'time' => DateTime.now.to_s})
      @logfile << msg.to_yaml
      @logfile.close
      raise ResetException
    end
    set_restrictive_permissions(@logfile_name)
  end
    
  def run
    @logfile = File.new(@logfile_name, 'a')
    ['SIGTERM','SIGHUP','SIGKILL'].each do |signal|
      trap(signal,@trapped)
    end
    trap "SIGUSR1", @reset
    msg = ['type' => 'startup', 'time' => DateTime.now.to_s]
    @logfile << msg.to_yaml
    @logfile.flush
    begin
      IO.popen("#{@urlsnarf} -i any 2> /dev/null") do |urlsnarf|
        urlsnarf.each do |line|
          line.chomp!
          url = line.gsub(/^.*(GET|POST) ([^\s]+)\s.*$/, '\2')
          time_str = line.gsub(/^[^\[]*\[([^\]]*)\].*$/, '\1')
          time_str.gsub!(/\//,' ').gsub!(/(\d{4,}):/,'\1 ')
          timestamp = DateTime.parse time_str
          data = My_Array.new
          data.push({'type' => 'url', 'url' => url,
                     'time' => timestamp.to_s}) 
          @logfile << data.to_yaml
          @logfile.flush
        end
      end
      raise Interrupt # We get here if urlsnarf dies prematurely
    rescue Interrupt
      msg = My_Array.new
      msg.push({'type' => 'shutdown', 'method' => 'Interrupt',
                'time' => DateTime.now.to_s})
      @logfile << msg.to_yaml
      @logfile.close
      if defined?(@pid_file) and @pid_file then File.delete @pid_file
      end
    end # begin..rescue
  end # def run
  
  private
  
  def set_restrictive_permissions(*files)
    files.each do |f|
      begin
        unless File.file?(f)
          File.open(f,'w') {|file| }
        end
          File.chown(0,0,f)
          File.chmod(0600, f)
      rescue Errno::EPERM
        raise PermissionsException, "You must be root to alter these permissions."
      end
    end
  end
end