raise "Can't run this file directly" unless defined? $lib_dir
# Net Responsibility
#
# Copyright (C) 2007 Scott Severance
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Author:
#	Scott Severance <http://www.scottseverance.us>

require "#{$lib_dir}logfile"
require "#{$lib_dir}options"
require 'yaml'

class Report
  def initialize(options = nil)
    options = Options.new unless options
    @options = options
    @logfile = Logfile.new @options
    @dates = @logfile.sort_events_by_date.keys
  end
  
  def url_report(dates = @dates)
    # FIXME: See FIXME below for more
    @logfile = Logfile.new @options
    output = ""
    dates.each do |date|
      output += "%s\n%s\n\n" % [date, "=" * date.to_s.length]
      evts_by_date = @logfile.sort_events_by_date(date) 
      evts_by_type = @logfile.sort_events_by_type(evts_by_date)
      urls = @logfile.sort_events_by_hostname(evts_by_type['url'])
      urls.each do |host|
        hostname = host[0]
        output += "    #{hostname}\n"
        host[1].each do |hit|
          output += "        Time: %s\n" % get_time(hit['date'])
          output += "        Path: %s\n\n" %hit['path']
        end
      end
      output += "\n"
    end
    output
  end
  
  def startup_report(dates = @dates)
    event_report("startup",dates)
  end
  
  def shutdown_report(dates = @dates)
    event_report("shutdown",dates)
  end
  
  def reset_report(dates = @dates)
    event_report('reset', dates)
  end
  
  def full_report(dates = @dates)
    starts  = make_header('Startup Report')
    starts += indent(startup_report(dates))
    
    shuts  = make_header("Shutdown Report")
    shuts += indent(shutdown_report(dates))
    
    resets  = make_header("Reset Report")
    resets += indent(reset_report(dates))
    
    urls  = make_header('URL Report')
    urls += indent(url_report(dates))
    
    starts + shuts + resets + urls
  end
  
  private

  def event_report(event,dates = @dates)
    # FIXME: This is really ugly. I shouldn't have to call Logfile.new
    # here, but I can't seem to be able to get any events by type if
    # I don't.
    # 
    # The bug appears to be in Logfile#sort_events_by_type, though I
    # can't figure out what's wrong with that method.
    @logfile = Logfile.new @options
    output = ""
    dates.each do |date|
      evts_by_date = @logfile.sort_events_by_date(date)
      evts_by_type = @logfile.sort_events_by_type(evts_by_date)
      starts = evts_by_type[event]
      next if starts == nil
      output += "%s\n%s\n" % [date, "=" * date.to_s.length]
      #print starts.to_yaml
      #t = {'starts' => starts}
      #p t
      starts.each do |start|
        case event
          when /startup|reset/
            output += "    %s: %s\n" % [get_time(start[0][1]),event.capitalize]
          when "shutdown"
            output += "    %s: %s\n" % [get_time(start[1][1]),start[0][1]]
          else
            raise "I don't know how to handle the event #{event}."
        end
      end
      output += "\n"
    end
    output + "\n"
  end
  
  def get_time(time)
    time.to_s.gsub(/^[\S]+ ([0-9:]+) .*$/,'\1')
  end
  
  def indent(str)
    tmp = ""
    str.each_line { |line| tmp += "    #{line}" }
    tmp
  end
  
  def make_header(str)
    line = "#" * (str.length + 6)
    title = "## %s ##" % str
    "%s\n%s\n%s\n\n" % [line,title,line]
  end
end